@@ -15,15 +15,9 @@ import logging
 import re
 log = logging.getLogger()
 
-# Try to import keras resources from tensorflow
-try:
-    from tensorflow.keras.utils import Sequence
-    from tensorflow.keras.preprocessing.image import \
-        ImageDataGenerator, img_to_array
-except ModuleNotFoundError:
-    from keras.utils import Sequence
-    from keras.preprocessing.image import \
-        ImageDataGenerator, img_to_array
+from keras.utils import Sequence
+from keras.preprocessing.image import \
+    ImageDataGenerator, img_to_array
 
 
 class VideoFrameGenerator(Sequence):
@@ -199,7 +193,8 @@ class VideoFrameGenerator(Sequence):
         self.classes_count = len(classes)
 
         # to initialize transformations and shuffle indices
-        self.on_epoch_end()
+        if 'no_epoch_at_init' not in kwargs:
+            self.on_epoch_end()
 
         kind = "train"
         if _validation_data is not None:
@@ -391,7 +386,10 @@ class VideoFrameGenerator(Sequence):
     def _get_frames(self, video, nbframe, shape, force_no_headers=False):
         cap = cv.VideoCapture(video)
         total_frames = self.count_frames(cap, video, force_no_headers)
-        frame_step = floor(total_frames/nbframe/2)
+        orig_total = total_frames
+        if total_frames % 2 != 0:
+            total_frames += 1
+        frame_step = floor(total_frames/(nbframe-1))
         # TODO: fix that, a tiny video can have a frame_step that is
         # under 1
         frame_step = max(1, frame_step)
@@ -404,7 +402,7 @@ class VideoFrameGenerator(Sequence):
                 break
 
             frame_i += 1
-            if frame_i % frame_step == 0:
+            if frame_i == 1 or frame_i % frame_step == 0 or frame_i == orig_total:
                 # resize
                 frame = cv.resize(frame, shape)
 
